c Generate parameters needed by QTCM model and produce vertical profiles
cc   for temperature, humidity, horizontal and vertical velocities,...
c
c  a1(p): vertical profile of perturbed temperature
c  a1p(p): a1^+
c  a1hat: a1 vertical average
c  V1(p): vertical profile of baroclinic velocity(v1); = a1^+ - <a1^+>
c          where <>denotes vertical average and ^+ is {dlnp
c  V12hat: vertical average of V1^2
c  V1s:   V1 at surface
c  Omega1(p): vertical profile of vertical velocity(omega); = {V1dp
c  b1(p): vertical humidity for preturbed humidity
c  b1hat: vertical average of b1
c  b1s:   b1 at surface, for evaporation
c  eps_1:  damping for v1
c  eps_0:  damping for v0(barotropic mode)
c  eps_01: cross damping due to v0 on v1
c  GMsr:  ref. state gross moist static energy vertically averaged;=<V1sref>
c  GMsp:  x T1 is the GMs perturbation due to T1
c  GMqr:  ref. state gross moist static energy vertically averaged;=<V1qref>
c  GMqp:  x T1 is the GMs perturbation due to q1

c  dp=1.e2      !data increment is 1mb=100pa
c  pt: cloud top
c  pb: cloud base
c  p0: surface pressure
c  pressure cooradinate:
c                       i=0: surface layer
c                       i=npb: cloud base
c                       i=npt: cloud top

        parameter (p0=100000.,pt=20000.,pb=90000.,dp=100.,delp=p0-pt)
	parameter (nz=p0/100.-1,npt=(p0-pt)/100.,npb=(p0-pb)/100.)
	parameter (nvmod=2,nTmod=1)
	real a1(0:npt),a1p(0:npt),V1(0:npt),Omega1(0:npt),b1(0:npt)
	real V2(0:npt),Omega2(0:npt)
	real zp(0:npt),zz(0:nz),V1z(0:nz)
	real visc(0:npt),sref(0:npt),Tref(0:npt),qref(0:npt),qcref(0:npt)
        real gamma(0:npt),Delta(0:npt),alphaf(0:npt)
	real Tsat(0:npt),qsat(0:npt),ssat(0:npt)
	real dpV1(0:npt),dpV2(0:npt)
	real tmp(0:npt),tmp1(0:npt)
	real V(0:nvmod-1,0:npt),Vsqhat(0:nvmod-1)
	real Omega(0:nvmod-1,0:npt),dpV(0:nvmod-1,0:npt)
	real Vijk(0:nvmod-1,0:nvmod-1,0:nvmod-1),tmp8(0:npt)
        real Vwijk(0:nvmod-1,0:nvmod-1,0:nvmod-1),tmp9(0:npt)
	real DTijk(0:nvmod-1,nTmod,nTmod)
	real Dqijk(0:nvmod-1,nTmod,nTmod)
        real qrefhat,bb1(0:npt),bb1hat

c Physical contants, all in SI units 
	gravity=9.8
	Rair=287.04
	Cp=1004.
	rhoair=1.2
	Hlatent=2.43e6      !for T=30C, from Fleagle and Businger

c model dependent parameters
	CD=1.4e-3
	Vs=10.          !m/s
	CV0=CD*Vs
	alpha=.834       !relative humidity for Betts-Miller

c Delta: Hygrometric constant modified
	do i=0,npt
	    p=p0-i*dp
	    Delta(i)=Hlatent/Cp*1.e5/p  !hsat,hsats is for p=1000mb
            sigma=(p-pt)/(p0-pt)
            alphaf(i)=0.6+0.25*sigma    !linear fit to observed RH
	end do


	open(12,file='Tqo3.ref',status='old')
	open(21,file='qtcmpar.in',status='unknown')
	open(31,file='vtpar.in',status='unknown')
	open(22,file='prof.out',status='unknown')
	open(23,file='par.out',status='unknown')
        open(24,file='V1z',status='unknown')
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Tq.soundings: profiles of some warm place
      read(12,*)Ts
      do i=0,npt
        read(12,*)Tref(i),qref(i)
	qref(i)=qref(i)*Hlatent/Cp    !convert humidity to Kelvin
      enddo
      Trefhat=zbar(tref,npt)
      qrefhat=zbar(qref,npt)

c- moist adiabat, given temperature at p0
c  Approximate pressure in Delta from current level,elsewhere middle value
c    while temperature is from level below(small error since dp/dT small)
	Tsat(0)=Tref(0)        !Temperature at p0
	qsat(0)=Delta(0)*hsat(Tsat(0))
        qcref(0)=alphaf(0)*qsat(0)
c below cloud base (npb), Tsat is following dry adiabatic
        do k=1,npb
            pp=p0-(k-.5)*dp
            Tsat(k)=Tsat(0)*(pp/p0)**(Rair/Cp)
            qsat(k)=Delta(k)*hsat(Tsat(k))
            qcref(k)=alphaf(k)*qsat(k)
        enddo
c above cloud base (npb), Tsat is following moist adiabatic
        do i=npb+1,npt
	    p=p0-(i-.5)*dp
	    tt=Tsat(i-1)
	    Tsat(i)=tt+tt/(1.+Delta(i)*hsats(tt))
     &            *(Rair/Cp+Delta(i)*hsat(tt)/tt)*(-dp)/p
	    qsat(i)=Delta(i)*hsat(Tsat(i))
            qcref(i)=alphaf(i)*qsat(i)
	end do
c- from the oberved reference profile (mean) in Tqo3.ref
c       cloud top at npt from hb=ht
        Tcrefhat=zbar(Tsat,npt)
        qcrefhat=zbar(qcref,npt)
        Tcrefs=Tsat(0)
        qcrefs=qcref(0)

c- calculate a1 using Tsat
	do i=0,npt
	    p=p0-(i-1)*dp
	    gamma(i)=Delta(i)*hsats(Tsat(i))
	end do
       
	tmp1(npb)=0.
	do i=npb,npt-1
	    p=p0-(i-.5)*dp
	    tmp1(i+1)=tmp1(i)+1./(1.+.5*(gamma(i)+gamma(i+1)))*dp/p
	end do
	do i=npb,npt
	    tmp1(i)=1./(1.+gamma(i))*exp(-Rair/Cp*tmp1(i))
	    a1(i)=tmp1(i)
	end do
        do k=0,npb
	    p=p0-(k-.5)*dp
            a1(k)=(1./(1.+gamma(npb)))
     *         *(p/pb)**(Rair/Cp)
        enddo


c- a1hat,a1s
	a1hat=zbar(a1,npt)
	a1s=a1(0)

c- a1p(p)
	a1p(0)=0.
	do i=0,npt-1
	    p=p0-(i-.5)*dp
	    a1p(i+1)=a1p(i)+.5*(a1(i)+a1(i+1))*dp/p !medium value
	end do
	a1phat=zbar(a1p,npt)

c- V1(p),V12bar,V1s
c        if(pt .lt. 20000.)then
c          npt1=(p0-20000.)/100.
c	  do i=0,npt1
c	   V1v(i)=a1p(i)-a1phat
c	  end do
c          do i=npt1+1,npt
c	   V1v(i)=V1v(npt1+npt1-i)
c          enddo
c        else
c	  do i=0,npt
c	   V1v(i)=a1p(i)-a1phat
c	  end do
c        endif
	do i=0,npt
	   V1(i)=a1p(i)-a1phat
	   tmp(i)=V1(i)**2
	end do
              
        
	V12hat=zbar(tmp,npt)
	V1s=V1(0)

c- Omega1(p)
	Omega1(0)=0.
	do i=0,npt-1
	    Omega1(i+1)=Omega1(i)+.5*(V1(i)+V1(i+1))*dp !approx. by medium value
	end do
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCc
c- B1s=b1s
	do i=0,npt
         bb1(i)=alphaf(i)*gamma(i)*a1(i)
	end do
  	bb1hat=zbar(bb1,npt)  ! bb1hat
        do i=0,npt
          b1(i)=qref(i)/qref(0)
        enddo
          b1s=b1(0)
          b1hat=zbar(b1,npt)
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c-- V2
	pvscal=.5e4           ! using the same profile for Vb as for viscosity
	pbl=8.e4-pvscal
	do i=0,npt
	    p=p0-(i-1)*dp
	    tmp(i)=.6+.5*tanh((p-pbl)/pvscal)
	    tmp1(i)=tmp(i)*V1(i)
	end do
	Vbhat=zbar(tmp,npt)
	tt1=zbar(tmp1,npt)
	do i=0,npt
	    V2(i)=tmp(i)-Vbhat-V1(i)*tt1/V12hat
	end do
	V2s=V2(0)
        do i=0,npt
	tmp(i)=V2(i)**2
        enddo
	V22hat=zbar(tmp,npt)

c- Omega2(p)
	Omega2(0)=0.
	do i=0,npt-1
	    Omega2(i+1)=Omega2(i)+.5*(V2(i)+V2(i+1))*dp !approx. by medium value
	end do

c- Interior viscosity; drops like tanh.
c ;assume isothermal so that rho(p)~p
	visc0=50.            !kinematic viscocity at surface; m^2/s
	do i=0,npt
	    p=p0-(i-1)*dp
	    visc(i)=visc0*(.6+.5*tanh((p-pbl)/pvscal))
	end do


c- eps_0,eps_01,eps_1,eps_10
	eps_0=gravity/delp*rhoair*CD*Vs
	eps_10=eps_0*V1s
	eps_20=eps_0*V2s

	eps_01=eps_0*V1s/V12hat
c direct way of calculating eps_1; since dpV1=-a1/p, a simpler form is used
	do i=0,npt-1
	    p=p0-(i-1)*dp
	    dpV2(i)=(V2(i)-V2(i+1))/dp
	    dpV1(i)=(V1(i)-V1(i+1))/dp
	    tmp1(i)=p
	end do
	    dpV1(npt)=dpV1(npt-1)       !approximate at the top
	    dpV2(npt)=dpV2(npt-1)       !approximate at the top
	    tmp1(npt)=p-dp
        do i=0,npt
	tmp(i)=tmp1(i)**2*visc(i)*dpV1(i)*dpV1(i)
        enddo
	temp=(gravity*rhoair/p0)**2*zbar(tmp,npt)
c          write(*,*)'2nd term  new cal   =',temp/V12hat

c ... since dpV1=-a1/p, this is used instead
        do i=0,npt
	tmp(i)=visc(i)*a1(i)**2
        enddo
	temp=(gravity*rhoair/p0)**2*zbar(tmp,npt)
	eps_1=eps_0*V1s**2/V12hat + temp/V12hat
c	  write(*,*)'1st term in eps_1=',eps_0*V1s**2/V12hat
c          write(*,*)'2nd term    =',temp/V12hat

        do i=0,npt
	tmp(i)=tmp1(i)**2*visc(i)*dpV2(i)*dpV1(i)
        enddo
	temp=(gravity*rhoair/p0)**2*zbar(tmp,npt)
	eps_21=eps_0*V1s*V2s/V12hat+temp/V12hat

	eps_02=eps_0*V2s/V22hat
	eps_12=eps_0*V1s*V2s/V22hat+temp/V22hat !temp as the same as for eps_21
        do i=0,npt
	tmp(i)=tmp1(i)**2*visc(i)*dpV2(i)*dpV2(i)
        enddo
	temp=(gravity*rhoair/p0)**2*zbar(tmp,npt)
	eps_2=eps_0*V2s**2/V22hat+temp/V22hat

c- ssat(p),sref(p),GMsr,GMsp,GMqr,GMqp
	ph=0.
	phref=0.
	do i=0,npt-1
	    ssat(i)=Tsat(i)+ph/Cp
	    sref(i)=Tref(i)+phref/Cp
	    p=p0-(i-1)*dp
	    ph=ph+Rair*.5*(Tsat(i)+Tsat(i+1))*dp/p
	    phref=ph+Rair*.5*(Tref(i)+Tref(i+1))*dp/p
	end do
	ssat(npt)=Tsat(npt)+ph/Cp
	sref(npt)=Tref(npt)+phref/Cp
	GMsr=zbar2(V1,sref,npt)
        do i=0,npt
	tmp(i)=a1(i)+a1p(i)*Rair/Cp
        enddo
	GMsp=zbar2(V1,tmp,npt)
	GMqr=-zbar2(V1,qref,npt)
	GMqp=-zbar2(V1,b1,npt)
c	GMqp=-zbar2(V1,b,npt)
c- forcing GMsr and GMqr to be a fixed value because of numberical 
c  instability
        GMsr=3.5
        GMqr=3.0

c- V0: divergence due to topographic lifting
c       GMs0r=sref(0)+zbar(sref,npt)
c       GMq0r=qref(0)+zbar(qref,npt)
        GMs0r=sref(npt)-sref(0)
        GMq0r=qref(npt)-qref(0)

c- PBL mode V2 (@not used)
	GMs2r=zbar2(V2,sref,npt)
        do i=0,npt
	tmp(i)=a1(i)+a1p(i)*Rair/Cp
        enddo
	GMs2p=zbar2(V2,tmp,npt)
	GMq2r=zbar2(V2,qref,npt)
	GMq2p=zbar2(V2,b1,npt)

        do i=0,npt
	V(0,i)=1.
        enddo
	Vsqhat(0)=1.
        do i=0,npt
	V(1,i)=V1(i)
        enddo
	Vsqhat(1)=V12hat
	
	do k=0,nvmod-1     !some redundance here(Vijk is somewhat symmetric)
	do j=0,nvmod-1
	do i=0,nvmod-1
            do ii=0,npt
	    tmp8(ii)=V(i,ii)*V(j,ii)*V(k,ii)
            enddo
	    Vijk(i,j,k)=zbar(tmp8,npt)/Vsqhat(k)
	    if(Vijk(i,j,k).lt.1.e-5) Vijk(i,j,k)=0.      !force small # zero
	end do
	end do
	end do

        do i=0,npt
         p=p0-i*dp
         Omega(0,i)=p-pt
         Omega(1,i)=Omega1(i)
         dpV(0,i)=0
         dpV(1,i)=dpV1(i)
        enddo
          
        do k=0,nvmod-1     
        do j=0,nvmod-1
        do i=0,nvmod-1
            do ii=0,npt
            tmp9(ii)=V(k,ii)*Omega(i,ii)*dpV(j,ii)
            enddo
            Vwijk(i,j,k)=zbar(tmp9,npt)/Vsqhat(k)
        end do
        end do
        end do


	do k=1,nTmod
	do j=1,nTmod
	do i=0,nvmod-1
            do ii=0,npt
	    tmp8(ii)=V(i,ii)*a1(ii)
            enddo
	    DTijk(i,j,k)=zbar(tmp8,npt)/a1hat
            do ii=0,npt
	    tmp8(ii)=V(i,ii)*b1(ii)
            enddo
	    Dqijk(i,j,k)=zbar(tmp8,npt)/b1hat
            abc=zbar(tmp8,npt)
	end do
	end do
	end do
c---
c	Dqijk(1,1,1)=-6.5869723760904D-02
c---
        do kk=0,npt,50
c           print *,kk,V(1,kk),b1(kk)
        enddo

	eps_c=1./7200.          !Betts-Miller adjustment time scale
	
	Cpg=Cp*delp/gravity     !conversion between column heating and flux
cc- E, H ....
	eps_q=Cp*rhoair*CD*Vs   !evaporation        !W/m^2/K
	eps_s=Cp*rhoair*CD*Vs   !sensible heat      !W/m^2/K

c   (old)Jan(?;do annual later) Renolds SST average between 30N to 30S
c use a typical warm pool value
	Tsref=Ts
c- Soil heat Capacity x Depth; assume .1m water-like soil
	slCD=4.18e3*1.e3*.1       !1cal/g/K * 1g/cm3 * .1m

	qrefs=qref(0)
	Trefs=Tref(0)

c write out parameters at levels 1000 mb to 850 mb, at 50 mb interval
        do i=0,150,50
            p=p0/100.-float(i)
            write(23,*) V1(i),a1(i),bb1(i),b1(i),-Omega1(i),
     &                  Tref(i),qref(i)
            write(31,*) p,alphaf(i),Tsat(i),a1(i)
        end do
c write out parameters at levels 800 - 200 mb
        do i=200,npt,100
            p=p0/100.-float(i)
            write(23,*) V1(i),a1(i),bb1(i),b1(i),-Omega1(i),
     &                  Tref(i),qref(i)
            write(31,*) p,alphaf(i),Tsat(i),a1(i)
        end do
            write(31,*) a1hat

c- get V1 on z coordinate by linear interpolation
	ph=0.
	do i=0,npt
	    zp(i)=ph/gravity     !geopotential height
	    p=p0-(i-1)*dp
	    ph=ph+Rair*Tref(i)*dp/p
	end do
	dz=10000/(nz+1)        !10km/nz
	do i=0,nz
	    zz(i)=(i-1)*dz
	end do

        call lintp(npt,zp,V1,nz,zz,V1z)
	do i=0,nz,100
	    write(24,*) V1z(i)
        end do
cccccccccccccccccccc
        do i=650,npt
            p=1000-i
            hhh=zp(i)*gravity+(qref(i)+Tref(i))*Cp
            hhh0=zp(50)*gravity+(Tref(50)+qref(50))*Cp
        enddo
cccccccccccccccccccc
c---------------------------


	do i=0,npt
	    write(22,*) a1(i),V1(i),-Omega1(i)*1.e-3,b1(i)
     &       ,Tsat(i),ssat(i),qsat(i),ssat(i)+qsat(i)
     &       ,Tref(i),sref(i),qref(i),sref(i)+qref(i)
     &       ,visc(i),visc(i)*a1(i)**2,V2(i),-Omega2(i)*1.e-3
	end do

	write(21,*) a1hat,'        a1hat    dimensionless'
	write(21,*) a1s,  '        a1s'
	write(21,*) V1s,  '        V1s'
c       write(21,*) V2s,  '        V2s'
	write(21,*) b1hat,'        b1hat'
	write(21,*) b1s,  '        b1s'
	write(21,*) bb1hat,  '        bb1hat'
c       GMsr=0.9                 !using the estimation of Yu,Chou,Neelin (1997)
c       GMqr=-0.7
	write(21,*) GMsr, '        GMsr',sqrt(GMsr*Rair/a1hat),'=c'
	write(21,*) GMsp, '        GMsp'
	write(21,*) GMqr, '        GMqr'
c       write(21,*) GMqr, '        GMqr',sqrt((GMsr+GMqr)*Rair/a1hat),'=cm'
	write(21,*) GMqp, '        GMqp'
	write(21,*) GMs0r, '        GMs0r'
	write(21,*) GMq0r, '        GMq0r'
c       write(21,*) GMs2r, '        GMs2r'
c       write(21,*) GMs2p, '        GMs2p'
c       write(21,*) GMq2r, '        GMq2r'
c       write(21,*) GMq2p, '        GMq2p'
	write(21,*) eps_c, '        eps_c   /s'
	write(21,*) CV0, '        CV0'

cc- allow some stress at TOA, so zonal mean of total zonal wind can be nonzero
c HOwever, with advction this problem is gone. @Not used now
c     its contribution to V1,V2 mode is ignored for now
cept=eps_0*.05        !arbitrarily set TOA stress 1/20 of that at surface
ceps_0=eps_0+ept
ceps_10=eps_10+ept*V1(np)

	write(21,*) eps_0, '        eps_0 /s ->',1./(86400*eps_0),'day'
	write(21,*) eps_10,'        eps_10 ',1./(86400*eps_10),'day'
	write(21,*) eps_01,'        eps_01',1./(86400*eps_01),'day'
	write(21,*) eps_1, '        eps_1',1./(86400*eps_1),'day'
	write(21,*) Tsref, '        Tsref     K'
	write(21,*) Cpg, '        Cpg'
	write(21,*) Trefs, '        Trefs     K'
	write(21,*) qrefs, '        qrefs     K'
	write(21,*) Tcrefs, '       Tcrefs     K'
	write(21,*) qcrefs, '       qcrefs     K'
	write(21,*) Trefhat, '      Trefhat     K'
	write(21,*) qrefhat, '      qrefhat     K'
	write(21,*) Tcrefhat, '     Tcrefhat     K'
	write(21,*) qcrefhat, '     qcrefhat     K'
	write(21,*) Vijk,'         Vijk'
	write(21,*) Vwijk,'         Vwijk'
	write(21,*) DTijk,'        DTijk'
	write(21,*) Dqijk,'        Dqijk'
	write(21,*) (V1z(k),k=1,nz,100),'          V1(z)' 
	

	end
	


		function zbar2(a,b,n)
cc averaging of a times b
	real a(0:n),b(0:n)
	zbar2=0.
	do i=0,n
	    zbar2=zbar2+a(i)*b(i)
	end do
	zbar2=zbar2/(n+1)

	return
	end
		function zbar(a,n)
cc averaging a(0:n)
	real a(0:n)
	zbar=0.
	do i=0,n
	    zbar=zbar+a(i)
	end do
	zbar=zbar/(n+1)

	return
	end

                real*8 function zbar8(a,n)
cc averaging a(1:n)
        real*8 a(0:n)
        zbar8=0.d0
        do i=0,n
            zbar8=zbar8+a(i)
        end do
        zbar8=zbar8/(n+1)

        return
        end

                function hsat(T1)
        T=T1-273.15
c       T=T1
        esat=.6108d0*dexp(17.27d0*T/(237.3+T))  !from Shuttleworth,unit kpa (?)
        hsat=.622*esat/100.         ! unit= kg/kg, assuming Pressure=100kpa

        return
        end


                function hsats(T1)
c-- the slope of hsat, by analytically differentiating hsat
c       T=T1
        T=T1-273.15
        esats=2503.d0/(237.3+T)**2*dexp(17.27d0*T/(237.3+T))
        hsats=.622*esats/100.         ! unit= kg/kg, assuming Pressure=100kpa

        return
        end


		subroutine lintp(n1,x1,y1,n2,x2,y2)
c****     linear interpolation. Interpolate y1(x1(n1)) on to y2(x2(n2)) 
c****   if |x2|>|x1|, the result is not clear (no extrapolation at the end?)
c   n1  # of points in x1
c   x1  variable of input data array
c   y1  function of input data array, x1(n1) -> y1=y1( x1(n1) )
c   n2,x2,y2 for output

	integer n1,n2
	real x1(0:n1),y1(0:n1),x2(0:n2),y2(0:n2)

c   write(*,*)n1,n2
	if(x2(0).lt.x1(0).or.x2(n2).gt.x1(n1)) then
c	    write(*,*) 'terror,  |x2|>|x1|, need extropolation'
c    stop
	end if

	j=0
	do i2=0,n2
100         j=j+1
c   write(*,*)i2,j
	    if(x2(i2).lt.x1(j)) then
		j=j-1
		p=( x2(i2)-x1(j) ) / ( x1(j+1)-x1(j) ) 
		y2(i2)=p*y1(j+1)+(1.-p)*y1(j)
	      else if(j.gt.n1) then
	        y2(i2)=y1(n1)
	      else
		go to 100
	    end if
	end do
		
	return
	end
